library IEEE                    ;
use IEEE.STD_LOGIC_1164.ALL     ; 
use IEEE.STD_LOGIC_ARITH.ALL    ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;
use IEEE.MATH_REAL.ALL; -- Pour la fonction ceil notamment. 


entity ALU is
	port(
		clk    : std_logic ; 
		i_OP   : in std_logic_vector(2 downto 0)  ; 
		i_LOAD : in std_logic					  ; 
		i_A    : in std_logic_vector(15 downto 0) ; 
		i_B    : in std_logic_vector(15 downto 0) ; 
		i_sub  : in std_logic ; -- bit de contrôle qui permet de différencier une addition d'une soustraction. 
		q_R  : out std_logic_vector(15 downto 0)  ; 
		rdy  : out std_logic 
	) ; 
end entity ALU ; 

architecture behave of ALU is

--------------------
---- COMPONENTS ----
--------------------

	-- component adder is
		-- port(
			-- i_A : in std_logic_vector(15 downto 0) ; 
			-- i_B : in std_logic_vector(15 downto 0) ; 
			-- i_carry : in std_logic ; 
			-- q_add    : out std_logic_vector(15 downto 0) ;
			-- overflow : out std_logic 
		-- ) ; 
	-- end component adder ; 
	
	-- component multiplier is
		-- port(
			-- clk : in std_logic ; 
			-- i_LOAD : in std_logic ; 		
			-- i_A : in std_logic_vector(15 downto 0) ;
			-- i_B : in std_logic_vector(15 downto 0) ;
			-- q_mult : out std_logic_vector(15 downto 0) ;
			-- rdy : out std_logic ;
			-- overflow : out std_logic 
		-- ) ; 
	-- end component multiplier ;	
	
	-- component divisor is
		-- port(
			-- clk : in std_logic ; 
			-- i_LOAD : in std_logic ; 		
			-- i_A : in std_logic_vector(15 downto 0) ;
			-- i_B : in std_logic_vector(15 downto 0) ;
			-- q_mult : out std_logic_vector(15 downto 0) ;
			-- rdy : out std_logic ;
			-- overflow : out std_logic 
		-- ) ; 
	-- end component divisor ;	
	
	
	
	signal l_output_adder   : std_logic_vector(15 downto 0) ; 
	signal l_output_mult    : std_logic_vector(15 downto 0) ; 
	signal l_output_divisor : std_logic_vector(15 downto 0) ; 
	signal l_output_shifter : std_logic_vector(15 downto 0) ;
	
	signal l_not_A	       : std_logic_vector(15 downto 0) ; 
	signal l_not_B	       : std_logic_vector(15 downto 0) ; 
	signal l_AxorB         : std_logic_vector(15 downto 0) ;  
	signal l_AorB          : std_logic_vector(15 downto 0) ;  
	signal l_AandB         : std_logic_vector(15 downto 0) ;  
	
	signal l_output_mult_B : std_logic_vector(15 downto 0) ; 	
	signal l_rdy_mult : std_logic ; 
	signal l_rdy_div  : std_logic ; 
	


begin

	l_not_A <= not(i_A) ; 
	l_not_B <= not(i_B) ;
	l_AxorB <= i_A xor i_B ;
	l_AandB <= i_A and i_B ; 
	l_AorB  <= i_A or  i_B ; 
	
	------------------
	-- Multiplexers --
	------------------
	
	multiplexer_not : process(i_sub)
	begin
		if (i_sub='0') then 
			l_output_mult_B <= i_B ;
		else 
			l_output_mult_B <= l_not_B ; 
		end if ; 
	end process ;
	
	
	with i_OP select q_R <=
		l_output_adder   when "000",
		l_output_mult    when "001",
		l_output_divisor when "010",
		l_not_A          when "011",
		l_AorB		     when "100",
		l_AxorB			 when "101",
		l_AandB			 when "110",
		l_output_shifter when "111",
		(others=>'0') when others ; 
		
	with i_OP select rdy <=
		l_rdy_mult    when "001",
		l_rdy_div when "010",
		'1' when others ; 
		
		
		
	
	--------------------
	-- Instantiations --
	--------------------

	adder_inst_0 : entity work.adder
				   port map(
					i_A => i_A,
					i_B => l_output_mult_B,
					i_carry => i_sub,
					q_add => l_output_adder
					) ; 
					
	mult_inst_0 : entity work.multiplier
				  port map(
					clk    => clk, 
					i_LOAD => i_LOAD,
					i_A    => i_A,
					i_B    => i_B, 
					q_mult => l_output_mult,
					rdy    => l_rdy_mult
				  ) ; 
				  
				  
	div_inst_0 : entity work.divisor
	  port map(
		clk    => clk, 
		i_LOAD => i_LOAD,
		i_A    => i_A,
		i_B    => i_B, 
		q_mult => l_output_divisor,
		rdy    => l_rdy_div
	  ) ; 

	

end architecture ; 