library IEEE                    ;
use IEEE.STD_LOGIC_1164.ALL     ; 
use IEEE.STD_LOGIC_ARITH.ALL    ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;


entity comparator_16bits is

	port(
		i_A : in std_logic_vector(15 downto 0) ;
		i_B : in std_logic_vector(15 downto 0) ;
		q_eq : out std_logic ; 
		q_gr : out std_logic
	) ; 

end entity comparator_16bits ; 


architecture behave of comparator_16bits is

	component comparator_8bits is

		port(
			i_A : in std_logic_vector(7 downto 0) ;
			i_B : in std_logic_vector(7 downto 0) ;
			q_eq : out std_logic ; 
			q_gr : out std_logic
		) ; 

	end component comparator_8bits ; 
	
	signal l_A_MSB : std_logic_vector(7 downto 0) ;
	signal l_A_LSB : std_logic_vector(7 downto 0) ; 
	signal l_eq_MSB : std_logic ; 
	signal l_gr_MSB : std_logic ; 	

	signal l_B_MSB : std_logic_vector(7 downto 0) ;
	signal l_B_LSB : std_logic_vector(7 downto 0) ; 	
	signal l_eq_LSB : std_logic ; 
	signal l_gr_LSB : std_logic ; 		

begin

	l_A_MSB <= i_A(15 downto 8) ;
	l_A_LSB <= i_A(7 downto 0) ;
	l_B_MSB <= i_B(15 downto 8) ;
	l_B_LSB <= i_B(7 downto 0) ;	

	comparator_8bits_MSB : comparator_8bits
						   port map(
							i_A  => l_A_MSB,
							i_B  => l_B_MSB,
							q_eq => l_eq_MSB,
							q_gr => l_gr_MSB
						) ;
						
	comparator_8bits_LSB : comparator_8bits
						   port map(
							i_A  => l_A_LSB,
							i_B  => l_B_LSB,
							q_eq => l_eq_LSB,
							q_gr => l_gr_LSB
						) ;					

	q_eq <= l_eq_LSB and l_eq_MSB ; 
	q_gr <= l_gr_MSB or (l_eq_MSB and l_gr_LSB) ; 


end architecture behave ; 