library IEEE                    ;
use IEEE.STD_LOGIC_1164.ALL     ; 
use IEEE.STD_LOGIC_ARITH.ALL    ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;
use IEEE.MATH_REAL.ALL; -- Pour la fonction ceil notamment. 


entity multiplier is
	port(
		clk : in std_logic ; 
		i_LOAD : in std_logic ; 		
		i_A : in std_logic_vector(15 downto 0) ;
		i_B : in std_logic_vector(15 downto 0) ;
		q_mult : out std_logic_vector(15 downto 0) ;
		rdy : out std_logic ;
		overflow : out std_logic 
	) ; 
end entity multiplier ;


architecture behave of multiplier is

	constant zero : std_logic_vector(15 downto 0) := (others=>'0') ; 

	component adder is
		port(
			i_A : in std_logic_vector(15 downto 0) ; 
			i_B : in std_logic_vector(15 downto 0) ; 
			q_add    : out std_logic_vector(15 downto 0) ;
			overflow : out std_logic 
		) ; 
	end component adder ; 
	
	component is_zero is
		port(
			i_A : in std_logic_vector(15 downto 0) ;
			q_zero : out std_logic 
		) ;
	end component is_zero ; 	
	
	signal shf_register_0 : std_logic_vector(15 downto 0) := (others=>'0'); 
	signal shf_register_1 : std_logic_vector(15 downto 0) := (others=>'0'); 	
	signal acc_register   : std_logic_vector(15 downto 0) := (others=>'0');
	signal rdy_register   : std_logic := '0' ;
	
	signal tmp_0          : std_logic_vector(15 downto 0) ; 
	signal tmp_zero		  : std_logic ; 
	
	signal output_adder : std_logic_vector(15 downto 0) ; 
	
begin

	adder_inst_0 : adder
				   port map(
					i_A => tmp_0,
					i_B => acc_register,
					q_add => output_adder,
					overflow => overflow
				) ; 
				
	is_zero_inst_0 : is_zero
					 port map(
						i_A => shf_register_1,
						q_zero => tmp_zero
					) ; 

	-- multiplexer
	
	with shf_register_1(0) select
		tmp_0 <= shf_register_0 when '1',
				 zero when others ; 
				

	q_mult <= acc_register  ;
	rdy    <= rdy_register      ;
	process(clk)
	begin
		if rising_edge(clk) then
			acc_register <= output_adder ; -- mise à jour du registre d'accumulation. 
			rdy_register <= tmp_zero ; 
			if (i_LOAD='1') then
				shf_register_0 <= i_A ; 
				shf_register_1 <= i_B ; 
			else
				shf_register_0 <= shf_register_0(14 downto 0) & '0' ; 
				shf_register_1 <= '0' & shf_register_1(15 downto 1) ;
			end if ; 		
		end if; -- fin condition horloge. 
	end process ;


end architecture behave ; 